@@ -51,14 +51,14 @@ class TestSmtpAuth(MoxTestBase):
     def test_get_available_mechanisms(self):
         auth = Auth(None)
         self.assertEqual([CramMd5], auth.get_available_mechanisms())
-        self.assertEqual([Plain, Login, CramMd5],
+        self.assertEqual([CramMd5, Plain, Login],
                          auth.get_available_mechanisms(True))
 
     def test_str(self):
         auth = Auth(FakeSession(False))
         self.assertEqual('CRAM-MD5', str(auth))
         auth = Auth(FakeSession(True))
-        self.assertEqual('PLAIN LOGIN CRAM-MD5', str(auth))
+        self.assertEqual('CRAM-MD5 PLAIN LOGIN', str(auth))
 
     def test_unimplemented_means_invalid(self):
         auth = Auth(None)
@@ -152,5 +152,67 @@ class TestSmtpAuth(MoxTestBase):
         with self.assertRaises(ServerAuthError):
             auth.server_attempt(io, 'CRAM-MD5 dGVzdHVzZXI=')
 
+    def test_client_plain(self):
+        self.sock.sendall('AUTH PLAIN amtsAHRlc3RAZXhhbXBsZS5jb20AYXNkZg==\r\n')
+        self.sock.recv(IsA(int)).AndReturn('235 Ok\r\n')
+        self.mox.ReplayAll()
+        io = IO(self.sock)
+        reply = Plain.client_attempt(io, 'test@example.com', 'asdf', 'jkl')
+        self.assertEqual('235', reply.code)
+        self.assertEqual('2.0.0 Ok', reply.message)
+
+    def test_client_login(self):
+        self.sock.sendall('AUTH LOGIN\r\n')
+        self.sock.recv(IsA(int)).AndReturn('334 VXNlcm5hbWU6\r\n')
+        self.sock.sendall('dGVzdEBleGFtcGxlLmNvbQ==\r\n')
+        self.sock.recv(IsA(int)).AndReturn('334 UGFzc3dvcmQ6\r\n')
+        self.sock.sendall('YXNkZg==\r\n')
+        self.sock.recv(IsA(int)).AndReturn('235 Ok\r\n')
+        self.mox.ReplayAll()
+        io = IO(self.sock)
+        reply = Login.client_attempt(io, 'test@example.com', 'asdf', None)
+        self.assertEqual('235', reply.code)
+        self.assertEqual('2.0.0 Ok', reply.message)
+
+    def test_client_login_bad_mech(self):
+        self.sock.sendall('AUTH LOGIN\r\n')
+        self.sock.recv(IsA(int)).AndReturn('535 Nope!\r\n')
+        self.mox.ReplayAll()
+        io = IO(self.sock)
+        reply = Login.client_attempt(io, 'test@example.com', 'asdf', None)
+        self.assertEqual('535', reply.code)
+        self.assertEqual('5.0.0 Nope!', reply.message)
+
+    def test_client_login_bad_username(self):
+        self.sock.sendall('AUTH LOGIN\r\n')
+        self.sock.recv(IsA(int)).AndReturn('334 VXNlcm5hbWU6\r\n')
+        self.sock.sendall('dGVzdEBleGFtcGxlLmNvbQ==\r\n')
+        self.sock.recv(IsA(int)).AndReturn('535 Nope!\r\n')
+        self.mox.ReplayAll()
+        io = IO(self.sock)
+        reply = Login.client_attempt(io, 'test@example.com', 'asdf', None)
+        self.assertEqual('535', reply.code)
+        self.assertEqual('5.0.0 Nope!', reply.message)
+
+    def test_client_crammd5(self):
+        self.sock.sendall('AUTH CRAM-MD5\r\n')
+        self.sock.recv(IsA(int)).AndReturn('334 dGVzdCBjaGFsbGVuZ2U=\r\n')
+        self.sock.sendall('dGVzdEBleGFtcGxlLmNvbSA1Yzk1OTBjZGE3ZTgxMDY5Mzk2ZjhiYjlkMzU1MzE1Yg==\r\n')
+        self.sock.recv(IsA(int)).AndReturn('235 Ok\r\n')
+        self.mox.ReplayAll()
+        io = IO(self.sock)
+        reply = CramMd5.client_attempt(io, 'test@example.com', 'asdf', None)
+        self.assertEqual('235', reply.code)
+        self.assertEqual('2.0.0 Ok', reply.message)
+
+    def test_client_crammd5_bad_mech(self):
+        self.sock.sendall('AUTH CRAM-MD5\r\n')
+        self.sock.recv(IsA(int)).AndReturn('535 Nope!\r\n')
+        self.mox.ReplayAll()
+        io = IO(self.sock)
+        reply = CramMd5.client_attempt(io, 'test@example.com', 'asdf', None)
+        self.assertEqual('535', reply.code)
+        self.assertEqual('5.0.0 Nope!', reply.message)
+
 
 # vim:et:fdm=marker:sts=4:sw=4:ts=4
